/*
 * datastruct.h
 *
 *  Created on: Feb 1, 2011
 *      Author: Thomas PIERRARD
 */
# define PI 3.14159265358979
# define INF 1.0e14
# define GNUPLOT_COMMAND "gnuplot -persist"

/* structure of a list */
typedef struct lists {
	int index;
	int ranked;
	struct lists *parent;
	struct lists *child;
} list;

/* structure of an individual */
typedef struct {
	double constr_violation;
	double *xreal;
	int rank;
	double *obj;
	double *constr;
	double affinity;

} individual;

/* structure of a population */
typedef struct {
	individual *ind;
	int size;
} population;

/* structure which permits parameters storage */
typedef struct globalArgs_t {
	const char *probname;
	double seed;
	double st_gs;
	double mut_stg;
	double mut_stl;
	double nb_cl;
} globalArgs_t;

/* Variables of the algorithm */
extern int nreal;
extern int nobj;
extern int ncon;
extern int popsize;
extern int ngen;
extern int curgen;
extern double st_gs;
extern double mut_stg;
extern double mut_stl;
extern double nb_cl;
extern int nbeval;
extern double *min_realvar;
extern double *max_realvar;
extern int *sort;

/* Variables of the random engine */
extern double seed;
extern double oldrand[55];
extern int jrand;

/* Variables for the parser */
extern void * plugin;
extern int retval;
extern char* prob_name;
extern globalArgs_t globalArgs;

/* necessary because popen and pclose are not ansi functions */
extern FILE *popen(const char *command, const char *modes);
extern int pclose(FILE *stream);

/* necessary to load plugin */
typedef double PROBLEM(double *xreal, double *obj, double *constr);
PROBLEM * test_problem;

/* affin.c : Define affinity */
void normalize_affinity(population * pop, int set);
void define_affinity(population *antg, population *antb);

/* alloc.c : Memory allocation */
void allocate_memory_pop(population *pop, int size);
void allocate_memory_ind(individual *ind);
void deallocate_memory_pop(population *pop, int size);
void deallocate_memory_ind(individual *ind);

/* clmut.c : Clone and mutation */
void clone_pop(population * antb, population * antg, population * pool);
void mutat_pop(population * pop);
void mutat_ind_gauss_global(individual *ind);
void mutat_ind_gauss_local(individual *ind);
void real_mutate_ind(individual *ind);
/* debug.c : Display useful information */
void display_pop(population * pop, int start, int end, int curr,
		const char * str);
void display_pop_sorted(population * pop, int start, int end, int curr,
		const char * str);

/* discd.c : Discard individuals to come back to a population of size N */
void normalize(population * pop);
void discard(population * pop, population *pool);
void discard2(population * pop, population *pool, population *antg);
/* domin.c : Dominance of an individal */
int check_domin_strict(individual * a, individual * b);
int dominate(individual *a, individual *b);

/* evalu.c : Evaluate a population */
void evaluate_pop(population *pop);
void evaluate_ind(individual *ind);

/* funct.c : Auxilliary functions */
float box_muller(float m, float s);
double eucl_d(individual * a, individual * b);
double absreal(double a);
int same(individual * a, individual * b);
void swap(int *x, int *y);
void copy_ind(individual *ind1, individual *ind2);
double max(double a, double b);
double min(double a, double b);

/* gplot.c : Routine to create plots via gnuplot */
void display_all_pop();
void display_gen(int ii);

/* HV*.c files : Signature of the function needed ? */
double hv_compute(population * pop, double *reference, double *maximum,
		double *minimum, int size, int * sort, int without, int *nobj_p);

/* hyper.c : Routine to discard N-k solutions in a population of size N */
void hyperv(population * pop, int k);
void crowd(population * pop, int k);

/* initi.c :  Initialize the main population */
void initialize_pop(population *pop);
void initialize_ind(individual *ind);

/* llist.c : List functions */
void linsert(list *node, int x);
list* ldel(list *node);

/* merge.c : Merge two populations */
void merge(population * pop, population * antg);

/* ndoms.c : Non-dominated sorting */
int read_ind(int **front, population * pop);
int find_ndf(int * front1, int nofront1, int ** front2, population * pop);
void dominated_sort(population * pop);

/* output.c : Output result files */
void output_pop(population *pop, int ii, const char *str);
void output_pop2(population *pop, population *pop2, int ii, const char *str);


/* parse.c : Usage && options parser */
void display_usage();
int parseopts(int argc, char * argv[]);

/* qsort.c : Quick sort algorithms */
void myqsort(population *pop, int m, int n, int obj, int * sort);
void myqsort_constr(population *pop, int m, int n, int * sort);
void myqsort_rank(population *pop, int m, int n, int * sort);
void myqsort_affinity(population *pop, int m, int n, int * sort);
void myqsort_affinity_inv(population *pop, int m, int n, int * sort);

/* rando.c : Random engine */
void randomize(void);
void warmup_random(double seed);
void advance_random(void);
double randomperc(void);
int rnd(int low, int high);
double rndreal(double low, double high);
void generate_rnd(int k, int i, int j, int **res);

/* selec.c : Select individuals to be clone */
void select_ind(population *candidate, population *antb, population *antg);
void select_hypev(population *candidate, population *antb, population *antg);
void select_crowd(population *candidate, population *antb, population *antg);


/* split.c : Split main population in two */
void split(population * pop, population * antb, population * antg);
